#ifndef SIMPLE_PETRI
#define SIMPLE_PETRI
/*
  sim: simple petri net simulator developed using the SUIT toolkit 

  Copyright (C) 1993  Sunil Gupta

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  The author can be contacted by emailing cs89ssg@brunel.ac.uk
  before the end of August 1993, or via the newsgroup alt.bugs after
  August 1993.
*/

/************************************************************************/
/*									*/
/*	    Use of SUIT to fast prototype a Petri-net simulator		*/
/*									*/
/*			Author: Sunil Gupta				*/
/*									*/
/*									*/
/*    permission is given to distribute these sources for inclusion	*/
/*    in non-commercial products. Any changes to the original sources	*/
/*    must be marked as modified.					*/
/*									*/
/*    no guarantee is either implied or expressed about the quality	*/
/*    suitability or stability  of the software.			*/
/*									*/
/*    The author cannot be held accountable or responsible for any	*/
/*    loss or corruption of data associated with the use of this product*/
/*									*/
/*									*/
/*									*/
/*  Inception and creation of SUIT interface		Friday 16-10-92	*/
/*									*/
/*  implementation of 4way binary tree			Monday 19-10-92	*/
/*									*/
/*  implementation of Insertion into tree		Tuesday 20-10-92*/
/*									*/
/*  implementation of insertion if thing to be added			*/
/*  is closer to the parent of the object it was being 			*/
/*  attached to						Weds 21-10-92	*/
/*									*/
/*  added function header blocks			Fri 23-10-92	*/
/*									*/
/*  changed implementation so that nodes are moved 			*/
/*  rather than destroyed and recreated when changed			*/
/*  in the tree						Mon 26-10-92	*/
/*									*/
/*  added functions to link objects into a net		Mon 26-10-92	*/
/*									*/
/*  added functions to draw arrows on paths, thanks to			*/
/*  em90sct for help with maths				Thu 29-10-92	*/
/*									*/
/*  the point has been proven, 4 way binary trees are possible to do	*/
/*  but the resultant code is wayyy to complex. So changing strategy to	*/
/*  use a 2D sparse array with hashing			Fri 28-11-92	*/
/*									*/
/*  allow deletion of tokens				Sun 29-11-92	*/
/*									*/
/*  places and transitions painted after paths have been done		*/
/*  makes the result look uncluttered			Mon 31-11-92	*/
/*									*/
/*  added strenghts to paths to minimise storage requirements for	*/
/*  paths going between the same objects		Mon 31-11-92	*/
/*									*/
/*  error detected in the method of storing paths. only the transitions	*/
/*  need to know about them, not the paths as the transitions will	*/
/*  whether to fire or not. places are merely passive 	Tue 01-11-92	*/
/*									*/
/*  paths can be removed				Tue 01-11-92 	*/
/*									*/
/*  working on removal of transitions, but am having a few problems	*/
/*									*/
/*  Duncan Stansfield suggested that the removal of elements from btrees*/
/*  would be too difficult, how about just marking the elements as	*/
/*  being deleted and reorder the tree another time.	Wed 02-12-92	*/
/*									*/
/*  removing places using a similar technique to above. PLaces are 	*/
/*  only marked as deleted, when redrawing if a path is attached to	*/
/*  something that does not exist, it is deleted	Thu 03-12-92	*/
/*									*/
/*  added support for those pathetic mono screens	Thu 03-12-92	*/
/*									*/
/*  removing paths when drawing them causes a lot of seg faults. besides*/
/*  the painting part should only paint and nothing else		*/
/*									*/
/*  feedback from Mark Sonnen on the nature of interface has led to 	*/
/*  significant improvement				Fri 04-12-92	*/
/*									*/
/*  curently paths are not stored on places. This means a large amount	*/
/*  of computation is needed to work out which paths connect to which	*/
/*  places. According to my mentor PVS, doing analysis on this would be	*/
/*  difficult. tis true. so although the information appeared to be 	*/
/*  redundant, it was necc				Mon 07-12-92	*/
/*									*/
/*  removal of transitions and places with their associated paths is	*/
/*  fully working					Thu 10-12-92	*/
/*									*/
/*  adding labels to places and transitions is fully implemented	*/
/*							Thu 10-12-92	*/
/*									*/
/*  fixed bug in adding/removing places. Didnt check for objects that 	*/
/*  had been deleted					Fri 11-12-92	*/
/*									*/
/*  loading and saving of nets implemented		Fri 11-12-92	*/
/*									*/
/*  tracing and executing of net implemented		Thu 17-12-92	*/
/*									*/
/*  bugfix to net save routine, was causing only one path per object	*/
/*  to be saved						Mon 12-1-93	*/
/*									*/
/*  bugfix to save/load buttons. 3rd arg to AskForFilename was wrong	*/
/*  fixed radio buttons to have the correct ones activated at startup	*/
/*  added patchlevel.h					Tue 19-5-93	*/
/*									*/
/************************************************************************/

/************************************************************************/
/* Standard function header block					*/
/************************************************************************/

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/* Function name:							*/
/* 									*/
/* Description:								*/
/* 									*/
/* Date of creation:							*/
/* 									*/
/* input arguments:							*/
/* 									*/
/* output:								*/
/* 									*/
/* function return:							*/
/* 									*/
/* Modifications:							*/
/* 									*/
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

/************************************************************************/
/* INCLUDES								*/
/************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <suit.h>
#include "patchlevel.h"



/************************************************************************/
/* defines								*/
/************************************************************************/
#ifndef PI
#  define PI		3.141527
#endif

#define ILLEGAL_GRAD	0.008008008008
#define ARROW_WID	0.03
#define ARROW_HT	0.01
#define GRID		0.06
#define SYM_HT		(GRID/2.0)
#define ZAPALL		TRUE
#define ZAPONE		FALSE
#define SIG		"%%%%%%%PETRI_NET_FILE%%%%%%%"
#define MYBUFSIZ	1000


/************************************************************************/
/* type definitions							*/
/************************************************************************/

/* --------------------------------------------------------------------	*/
/* enumerated types							*/
/* --------------------------------------------------------------------	*/

typedef enum
{
  FIRED, DEAD
} Net_state;

typedef enum
{
    PLACE, TRANSITION, TOKEN, PATH, DELETED
} Object_type;

typedef enum { SOURCE, TARGET, BOTH, UNKNOWN } whatami;

typedef enum 
{ 
    ADD_PLACE, ADD_TRANS, ADD_PATH,  ADD_TOKEN,
    MOV_PLACE, MOV_TRANS, 
    DEL_PLACE, DEL_TRANS, DEL_PATH, DEL_TOKEN
} Action;

typedef enum
{
  ADDING, MOVING, ZAPPING
} Happening;


/* --------------------------------------------------------------------	*/
/* an object can be a  PLACE, TRANSITION, TOKEN or a path PATH 		*/
/*									*/
/* PLACES and TRANSITIONS are represented as hash-trees. Links are 	*/
/* represented as  pointers between the places and transitions. These	*/
/* are not stored explicitly as when places/trans are deleted, so are	*/
/* all paths associated with them.					*/
/* tokens are stored as counts within places.				*/
/* 									*/
/* spatial awareness is achieved by storing the objects in s 2d sparse	*/
/* array								*/
/* --------------------------------------------------------------------	*/


typedef struct blist      *b_list;
typedef struct path_list  *p_list;
typedef struct trans_list *t_list;

typedef struct path_list
{
  whatami	what_i_am;
  int		strength;
  b_list	item;
  p_list 	next;
}_path_list;

typedef struct blist
{
   int		hash;
   double	x, y;
   int		tokens;
   char		*label;
   Object_type	my_type;
   p_list	list;
   b_list	left, right;
}_blist;

typedef struct trans_list
{
    b_list	item;
    t_list	next;
}_trans_list;




/************************************************************************/
/* globals								*/
/************************************************************************/

extern SUIT_object obj_radio;
extern SUIT_object petri_obj;
extern SUIT_object status_obj;

extern char buffer[MYBUFSIZ];
extern char *filenam;

extern b_list  root;

extern int debug;
extern int show_grid;
extern int mono;
extern int ngrids;

extern int times_fired;

extern Happening objing;


/************************************************************************/
/* Prototypes								*/
/************************************************************************/

/*----------------------------------------------------------------------*/
/* main interface creating functions					*/
/*----------------------------------------------------------------------*/

void main(int argc, char *argv[]);
void setup_interface(void);

/*----------------------------------------------------------------------*/
/* call backs functions							*/
/*----------------------------------------------------------------------*/

void set_status(SUIT_object obj);
void obj_things(SUIT_object obj);

void clear_net(SUIT_object obj);
void execute_net(SUIT_object obj);
void refresh(SUIT_object obj);
void toggle_grid(SUIT_object obj);
void trace_net(SUIT_object obj);

void save_net(SUIT_object obj);
void load_net(SUIT_object obj);

/*----------------------------------------------------------------------*/
/* User defined  SUIT objects						*/
/*----------------------------------------------------------------------*/
SUIT_object CreatePetriWidget(char *name);
void	    HitPetriWidget (SUIT_object obj, SUIT_event ev);
void	    PaintPetriWidget (SUIT_object obj);
void 	    myPaintPetriWidget (SUIT_object obj);


/*----------------------------------------------------------------------*/
/* painting operations							*/
/*----------------------------------------------------------------------*/

void	paint_label( b_list branch );
void	paint_grid( SUIT_object obj );
void	paint_path_list( b_list node, p_list element);
void	paint_paths( b_list branch );
void	paint_symbols( b_list branch );
void	paint_triangle( GP_point p1, GP_point p2, GP_point p3);

/*----------------------------------------------------------------------*/
/*  create and destroy objects						*/
/*----------------------------------------------------------------------*/
b_list   create_blist_node( Object_type type, double x, double y);
void     destroy_blist(b_list *branch);

p_list	 create_plist_node(void);
void 	 destroy_plist(b_list branch, p_list *list);
void     destroy_plist_element
	 (
	    b_list branch, b_list target, whatami thing, int zapall
	 );

/*----------------------------------------------------------------------*/
/*  add/remove objects to the tree					*/
/*----------------------------------------------------------------------*/
void	add_blist_node( b_list *branch, b_list node);
void	add_path(b_list source, b_list target);
b_list	add_element( b_list *net, Object_type type, double x, double y);

void    destroy_path(b_list source, b_list target );
void	destroy_place( b_list node);	
void	destroy_transition(b_list node);	

/*----------------------------------------------------------------------*/
/*  vector functions							*/
/*----------------------------------------------------------------------*/

void    get_arrow(GP_point p1, GP_point p2,
		 GP_point *wing1, GP_point *nose , GP_point *wing2);
double	line_length( GP_point p1, GP_point p2);
void	midpoint( GP_point p1, GP_point p2, GP_point *mid);

/*----------------------------------------------------------------------*/
/*  load and save stuff							*/
/*----------------------------------------------------------------------*/
void save_branch( FILE *fp, b_list branch);
void save_header( FILE *fp);
void save_list( FILE *fp, b_list branch, p_list list);
void save_paths( FILE *fp, b_list branch);

int load_header( FILE *fp);
void load_data( FILE *fp, b_list *branch );

/*----------------------------------------------------------------------*/
/* execute the net							*/
/*----------------------------------------------------------------------*/
Net_state run_net(b_list branch);
int count_active( t_list list);
void find_active( b_list branch, t_list *list);
void execute_active(b_list branch, t_list list);
void destroy_active(t_list *list);

/*----------------------------------------------------------------------*/
/*  assorted functions							*/
/*----------------------------------------------------------------------*/
b_list    get_closest(b_list branch, double x, double y);
b_list    get_element(b_list branch, int hash_val);
int	  get_hash(double x, double y);
void unhash ( int hash, double *x, double *y);

void      set_colour( char *colour);

p_list	  is_path(b_list source, b_list target, whatami what);
p_list    is_wanted_path(p_list list, b_list element, whatami wanted);

char	  *itoa( int num);
int 	  SUIT_getNumber(char *prompt);
void 	  GNU_thing(void);

# if !defined(nint)
#  define nint(x) floor((x)+0.5)
# endif


#endif
